;===========================================================================================
; Midi1_In.asm  Fasm Flat assembler
; Programm par AsmGges France
;-------------------------------------------------------------------------------------------
; control class = id, pas de resource, controle total de CONTROL class
; .mid file     = id, resource
; Pour cette version MDI la prsence sur le disque du fichier .mid est indispensable pour
; la compilation du fichier .asm mais - PAS - pour l'excution du fichier .exe
; Aprs la compilation le fichier .exe est totalement autonome et le fichier .mid n'est plus
; ncessaire.
; Au dpart de l'excution du fichier .exe la resource contenue dans l'.exe est copie sur
; le disque dans un fichier temporaire temp.mid.
; A la fin du programme quand on quitte l'application ce fichier temporaire est effac.
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  ; Structures pour MCI
  ; -------------------
  struct OPEN_MCI
    dwCallback       dd ?
    wDeviceID        dd ?
    lpstrDeviceType  dd ?
    lpstrElementName dd ?
    lpstrAlias       dd ?
  ends

  struct GENERIC_CALLBACK
    dwCallback       dd ?
  ends

  struct PLAY_MCI
    dwCallback       dd ?
    dwFrom           dd ?
    dwTo             dd ?
  ends

  ; Equates pour MCI
  ; ----------------
  MCI_OPEN         = 0803h
  MCI_OPEN_ELEMENT = 0200h
  MCI_PLAY         = 0806h
  MCI_CLOSE        = 0804h

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

 start:
    ; Message demande attente traitement mci
    ; --------------------------------------
    invoke MessageBox,HWND_DESKTOP,LoadMsg,ClassName,MB_OK+MB_ICONINFORMATION

    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Dtermine l'emplacement de la ressource raw datas
    ; -------------------------------------------------
    invoke FindResource,[wc.hInstance],ID_MIDIDATAS,RT_RCDATA
    or eax,eax
    jz ErrMidi
    mov [hRes],eax

    ; Obtenir la taille de la ressource
    ; ---------------------------------
    invoke SizeofResource,[wc.hInstance],[hRes]
    or eax,eax
    jz ErrMidi
    mov [sizeRes],eax

    ; Charge la ressource en mmoire globale
    ; --------------------------------------
    invoke LoadResource,[wc.hInstance],[hRes]
    or eax,eax
    jz ErrMidi

    ; Verrouille la ressource en mmoire
    ; ----------------------------------
    invoke LockResource,eax
    or eax,eax
    jz ErrMidi
    mov [ptRes],eax

    ; Cration d'un fichier temporaire .mid
    ; -------------------------------------
    invoke CreateFile,TempFile,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0
    cmp eax,-1
    je  ErrMidi
    mov [hFile],eax

    ; Ecrit la ressource de l'.exe dans le fichier temporaire .mid
    ; ------------------------------------------------------------
    invoke WriteFile,[hFile],[ptRes],[sizeRes],ByteWrite,0
    or eax,eax
    jz ErrMidi

    ; Fermeture du fichier temporaire
    ; -------------------------------
    invoke CloseHandle,[hFile]
    or eax,eax
    jz ErrMidi

    ; Mci initialisation et chargement
    ; --------------------------------
    mov [open_mci.lpstrDeviceType],0
    mov [open_mci.lpstrElementName],TempFile
    invoke mciSendCommand,0,MCI_OPEN,MCI_OPEN_ELEMENT,open_mci
    or eax,eax
    jnz ErrMidi

    ; Mci excute le fichier .mid
    ; ---------------------------
    invoke mciSendCommand,[open_mci.wDeviceID],MCI_PLAY,0,play_mci
    or eax,eax
    jnz ErrMidi

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU+\
                          WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ErrMidi:
    ; Message d'erreur
    ; ----------------
    invoke MessageBox,HWND_DESKTOP,ErrMidiMsg,ClassName,MB_OK+MB_ICONINFORMATION

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Arret mci
        ; ---------
        invoke mciSendCommand,[open_mci.wDeviceID],MCI_CLOSE,0,generic_callback

        ; Efface le fichier temporaire .mid
        ; ---------------------------------
        invoke DeleteFile,TempFile

        ; Demande fin d'application
        ; -------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE
        ; Dfinir une fonte de texte
        ; --------------------------
        invoke CreateFont,8,0,0,0,FW_NORMAL,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                       CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                      MSSansSerif
        mov [hFontMSSS],eax                      ;fonte handle

        ; Dfinir une autre fonte en gras
        ; -------------------------------
        invoke CreateFont,24,0,0,0,FW_BOLD,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                      CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                         Verdana
        mov [hFont2],eax                         ;fonte handle

        ; Construit tous les controles
        ; ----------------------------
        stdcall CreateControls,[@hWnd]

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
proc CreateControls @hWnd

    ; Cration de la fenetre-enfant GroupBox
    ; --------------------------------------
    stdcall Dlg,Button,GbGges,[@hWnd],4,2,286,20,ID_GBOX,BS_GROUPBOX,0

    ;=======================================================================================
    ; Cration de la fenetre-enfant Edit
    ; ----------------------------------
    stdcall Dlg,Edit,FriendMsg,[@hWnd],15,42,264,146,ID_EDIT,ES_CENTER+WS_VSCROLL+\
                                           ES_MULTILINE+ES_READONLY+ES_WANTRETURN,\
                                                                 WS_EX_CLIENTEDGE
    ; Valider la fonte en gras pour Edit text control
    ; -----------------------------------------------
    invoke SendMessage,eax,WM_SETFONT,[hFont2],NULL

    ret
endp

;===========================================================================================
; Cration d'une fenetre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class    = adresse du nom de control class
; @AddrText = adresse du control text
; @hParent  = handle de la fenetre parent
; @X        = x-coordonne fenetre-enfant
; @Y        = y-coordonne fenetre-enfant
; @Xw       = largeur en pixels fenetre-enfant
; @Yh       = hauteur en pixels fenetre-enfant
; @Id       = identificateur du controle forme ID_XXX
; @Style    = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle  = window style tendu
;-------------------------------------------------------------------------------------------
; sortie eax = handle de la fenetre-enfant
;-------------------------------------------------------------------------------------------
proc Dlg @Class, @AddrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AddrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],NULL
    mov ebx,eax

    ; Valider la fonte standard MS Sans Serif pour le control
    ; -------------------------------------------------------
    invoke SendMessage,ebx,WM_SETFONT,[hFontMSSS],NULL
    mov eax,ebx

    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  Titre       db ' MIDI play',0
  ClassName   db ' AsmGges Win32',0
  MsgEnd      db ' AsmGges France @2006  ',0
  GbGges      db 'AsmGges MIDI',0
  LoadMsg     db "Aprs OK merci d'attendre la fin du traitement MIDI ",0
  FriendMsg   db 'Je vous envoie cette petite musique en vous souhaitant une bonne '
              db 'programmation',0
  ErrMidiMsg  db 'Erreur: mci',0
  NoText      db '',0

  MSSansSerif db 'MS Sans Serif',0
  Verdana     db 'Verdana',0

  Button      db 'BUTTON',0
  Edit        db 'EDIT',0

  TempFile    db 'temp.mid',0

  align 4

  WinX        dd 0
  WinY        dd 100
  WinW        dd 300
  WinH        dd 244

  hWnd        dd ?
  hFontMSSS   dd ?
  hFont2      dd ?

  hRes        dd ?
  sizeRes     dd ?
  ptRes       dd ?
  hFile       dd ?
  ByteWrite   dd ?

  wc          WNDCLASS  0, WindowProc, 0, 0, 0, 0, 0, COLOR_BTNFACE+1, 0, ClassName
  msg         MSG
  open_mci          OPEN_MCI
  generic_callback  GENERIC_CALLBACK
  play_mci          PLAY_MCI

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL',\
             winmm,'WINMM.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

  import winmm,\
         mciSendCommand,'mciSendCommandA'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons,\
            RT_RCDATA,datas

  IDI_FRANCE   = 17
  ID_MIDIDATAS = 20

  ID_GBOX      = 100
  ID_EDIT      = 101

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource datas,\
           ID_MIDIDATAS,LANG_NEUTRAL,MyMidi

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

  resdata MyMidi
      file 'res\avc960.mid'
  endres

;===========================================================================================
